/* Copyright (c) 2008-09 lvqcl.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */ 

#include "dsp_lpf_config.h"
const int STRMAXLEN=20;

t_dsp_lpf_params::t_dsp_lpf_params()
{
	cfg_.lp_freq        = 25000;
	cfg_.tbandw10       = dfltTBW10;
	cfg_.phase          = lpfPlinear;
}

void t_dsp_lpf_params::get_lpfconfig(lpfConfig& cfg) const
{
	cfg_.CloneTo(cfg);
}

bool t_dsp_lpf_params::set_data(const dsp_preset& p_data)
{
	t_int32 temp[3];
	if (p_data.get_owner() != g_get_guid()) return false;
	if (p_data.get_data_size() != sizeof(temp))
	{
		//from previous versions? -- in future
		return false;
	}
	memcpy(temp, p_data.get_data(), sizeof(temp));

	for(int i=0; i<3; i++)
		byte_order::order_le_to_native_t(temp[i]);
		
	cfg_.lp_freq        = temp[0];
	cfg_.tbandw10       = temp[1];
	cfg_.phase          = temp[2];

	return true;
}

bool t_dsp_lpf_params::get_data(dsp_preset& p_data) const
{
	t_int32 temp[3];
	temp[0] = cfg_.lp_freq;
	temp[1] = cfg_.tbandw10;
	temp[2] = cfg_.phase;

	p_data.set_owner(g_get_guid());
	for(int i=0; i<3; i++)
		byte_order::order_native_to_le_t(temp[i]);
	p_data.set_data(temp, sizeof(temp));
	return true;
}

bool lpfConfig::cannot_lowpass(unsigned samplerate) const
{
	if (samplerate/2 <= (unsigned)lp_freq) return true;
	return false;
}

void lpfConfig::CloneTo(lpfConfig& to) const
{
	to.lp_freq = lp_freq;
	to.tbandw10 = tbandw10;
	to.phase = phase;
}


void dialog_dsp_lpf::update_phresponseText(TCHAR* buf, int sz, CStatic phresponseText, int val)
{
	if (val==0) {
		phresponseText.SetWindowTextW(_T("Minimum"));
	}
	else if (val==50) {
		phresponseText.SetWindowTextW(_T("Linear"));
	}
	else {
		_sntprintf_s(buf, sz, _TRUNCATE, _T("%2d %%"), val);
		phresponseText.SetWindowTextW(buf);
	}
}


BOOL dialog_dsp_lpf::OnInitDialog(CWindow wndFocus, LPARAM lInitParam)
{
	m_hooks.AddDialogWithControls(m_hWnd);

	CTrackBarCtrl lpfreqSlider(GetDlgItem(IDC_LOWPASS));
	CTrackBarCtrl tbwSlider(GetDlgItem(IDC_TBW));
	CTrackBarCtrl phresponseSlider(GetDlgItem(IDC_LPHRESPONSE));

	CStatic lpfreqText(GetDlgItem(IDC_LPTEXT));
	CStatic tbwText(GetDlgItem(IDC_TBWTEXT));
	CStatic phresponseText(GetDlgItem(IDC_LPHASETEXT));

	lpfreqSlider.SetRange(0, 2500);
	lpfreqSlider.SetPos(params_.lp_freq()/10);
	lpfreqSlider.SetTicFreq(100);
	lpfreqSlider.SetPageSize(100);

	tbwSlider.SetRange(minTBW10, maxTBW10);
	tbwSlider.SetPos(params_.tbandw10());
	tbwSlider.SetTicFreq(10);
	tbwSlider.SetPageSize(10);

	phresponseSlider.SetRange(0, 50);
	phresponseSlider.SetPos(params_.phase());
	phresponseSlider.SetTicFreq(5);
	phresponseSlider.SetPageSize(5);

	update_display();

	return 0;
}

void dialog_dsp_lpf::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar pScrollBar)
{
	dirty_ = true;
	update_display();
}

void dialog_dsp_lpf::OnCommand(UINT uNotifyCode, int nID, CWindow wndCtl)
{
	switch (nID)
	{
	case IDOK:
		EndDialog(1);
		break;

	case IDCANCEL:
		// Data not changed
		callback_.on_preset_changed(old_data_);
		EndDialog(0);
		break;

	case IDC_LDEFAULTS:
		{
		CTrackBarCtrl lpfreqSlider(GetDlgItem(IDC_LOWPASS));
		CTrackBarCtrl tbwSlider(GetDlgItem(IDC_TBW));
		CTrackBarCtrl phresponseSlider(GetDlgItem(IDC_LPHRESPONSE));

		lpfreqSlider.SetPos(25000/10);
		tbwSlider.SetPos(50);
		phresponseSlider.SetPos(50);

		dirty_ = true;
		update_display();
		}
		break;
	}
}

void dialog_dsp_lpf::update_display()
{
	TCHAR buf[30];

	CTrackBarCtrl lpfreqSlider(GetDlgItem(IDC_LOWPASS));
	CTrackBarCtrl tbwSlider(GetDlgItem(IDC_TBW));
	CTrackBarCtrl phresponseSlider(GetDlgItem(IDC_LPHRESPONSE));

	params_.set_lp_freq(lpfreqSlider.GetPos()*10);
	params_.set_tbandw10(tbwSlider.GetPos());
	params_.set_phase(phresponseSlider.GetPos());

	if (dirty_)
	{
		dsp_preset_impl data;
		params_.get_data(data);
		callback_.on_preset_changed(data);
		// Data (potentially) changed
		dirty_ = false;
	}

	CStatic lpfreqText(GetDlgItem(IDC_LPTEXT));
	CStatic tbwText(GetDlgItem(IDC_TBWTEXT));
	CStatic phresponseText(GetDlgItem(IDC_LPHASETEXT));

	_sntprintf_s(buf, sizeof(buf)/sizeof(TCHAR), _TRUNCATE, _T("%2.1f %%"), (float(params_.tbandw10())/10.0));
	tbwText.SetWindowTextW(buf);

	_sntprintf_s(buf, sizeof(buf)/sizeof(TCHAR), _TRUNCATE, _T("%5.0f Hz"), float(params_.lp_freq()));
	lpfreqText.SetWindowTextW(buf);

	update_phresponseText(buf, sizeof(buf)/sizeof(TCHAR), phresponseText, params_.phase());


}
