/* Copyright (c) 2008-09 lvqcl.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */ 

#ifndef DSPLPFCONFIG_H
#define DSPLPFCONFIG_H

#include "../foobar2000/helpers/foobar2000+atl.h"
#include "../foobar2000/SDK/coreDarkMode.h"

#include "resource1.h"

enum LPQUAL
{
	LPQvery = 0,
	LPQhigh = 1,
	LPQmedium = 2,
};

const int lpfPminimum = 0, lpfPlinear = 50; //0...50%
const int dfltTBW10 = 50; //5%
const int minTBW10 = 10; //1%
const int maxTBW10 = 200; //20%

struct lpfConfig
{
	t_int32 lp_freq;
	t_int32 tbandw10;
	t_int32 phase;

	void CloneTo(lpfConfig& to) const;
	bool cannot_lowpass(unsigned in_samplerate) const;

};

class t_dsp_lpf_params
{
	//TCHAR str_[STRMAXLEN];
	lpfConfig cfg_;
public:
	t_dsp_lpf_params();
	t_dsp_lpf_params(const lpfConfig& cfg):cfg_(cfg){}
	void get_lpfconfig(lpfConfig& cfg) const;

	static const GUID &g_get_guid()
	{	// {9A88E143-A520-4ce2-917C-B5303E91ABEB}
		static const GUID guid = { 0x9a88e143, 0xa520, 0x4ce2, { 0x91, 0x7c, 0xb5, 0x30, 0x3e, 0x91, 0xab, 0xeb } };
		return guid;
	}

	//store data from preset
	bool set_data(const dsp_preset& p_data);
	//put data to preset
	bool get_data(dsp_preset& p_data) const;

	inline t_int32 lp_freq() const { return cfg_.lp_freq; }
	inline t_int32 tbandw10() const { return cfg_.tbandw10; }
	inline t_int32 phase() const { return cfg_.phase; }

	inline void set_lp_freq(t_int32 freq){ cfg_.lp_freq = freq; }
	inline void set_tbandw10(t_int32 pb){ cfg_.tbandw10 = pb; }
	inline void set_phase(t_int32 ph){ cfg_.phase = ph; }
};



class dialog_dsp_lpf : public CDialogImpl<dialog_dsp_lpf>
{
public:
	enum
	{
		IDD = IDD_LPF_CONFIG
	};
 
	BEGIN_MSG_MAP_EX(dialog_dsp_lpf)
		MSG_WM_INITDIALOG(OnInitDialog)
		MSG_WM_HSCROLL(OnHScroll)
		MSG_WM_COMMAND(OnCommand)
	END_MSG_MAP()

private:
    bool dirty_;
    const dsp_preset& old_data_;
	
	t_dsp_lpf_params params_;
	dsp_preset_edit_callback& callback_; 
	fb2k::CCoreDarkModeHooks m_hooks;

	void update_phresponseText(TCHAR* buf, int sz, CStatic phresponseText, int val);

public:
	dialog_dsp_lpf(const dsp_preset& p_data, dsp_preset_edit_callback & p_callback) : old_data_(p_data),
		callback_(p_callback), dirty_(false) { params_.set_data(old_data_); }

protected:
	BOOL OnInitDialog(CWindow wndFocus, LPARAM lInitParam);
	void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar pScrollBar);
	void OnCommand(UINT uNotifyCode, int nID, CWindow wndCtl);
    void update_display();

};

#endif
