/* interface to SoX routines
 *
 * Copyright (c) 2009-2011 lvqcl.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */ 

#ifndef LOWPASS_H
#define LOWPASS_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>

#undef sample_t
typedef double sample_t;

enum LPF_errors
{
	LPF_EOF = -1,
	LPF_OK = 0,
	LPF_ENOMEM = 1,
	LPF_INTERNAL,
	LPF_NULLHANDLE,
	LPF_SOXERROR
};

enum LPF_phase
{
	LPF_minimum = 0,
	LPF_intermediate = 25,
	LPF_linear = 50,
};

typedef struct LPF_handle_tag LPF_handle;

typedef struct LPF_config_tag
{
	int samplerate;
	int lp_freq;
	double bandwidth; //in percent
	double phase;
} LPF_config;

#if SIZE_MAX < UINT64_MAX
typedef float fb_sample_t;
#else
typedef double fb_sample_t;
#endif

extern LPF_handle* (*LPF_init)(const LPF_config* config, int nchannels, int *error); /*create+start*/
extern LPF_handle* (*LPF_close)(LPF_handle *h); /*stop*/
extern int (*LPF_flow)(LPF_handle *h, const fb_sample_t* ibuf, fb_sample_t* obuf, size_t isamp, size_t osamp, size_t* iused, size_t* ogen); /*flow*/
extern int (*LPF_drain)(LPF_handle *h, fb_sample_t* obuf, size_t osamp, size_t* ogen); /*drain*/

const char* LPF_strerror(int error);

void lpf_init0_SSE3();
void lpf_init0_generic();

#ifdef __cplusplus
}
#endif

#endif
