/* libSoX Internal header
 *
 *   This file is meant for libSoX internal use only
 *
 * Copyright 2001-2008 Chris Bagwell and SoX Contributors
 *
 * This source code is freely redistributable and may be used for
 * any purpose.  This copyright notice must be maintained.
 * Chris Bagwell And SoX Contributors are not responsible for
 * the consequences of using this software.
 * (C) 2008-09 lvqcl - some changes for thread safety (see struct thread_fft_cache)
 */

#ifndef SOX_I_H
#define SOX_I_H

#include "sox.h"
#include "util.h"
#include "fft4g.h"

double lsx_bessel_I_0(double x);
int lsx_set_dft_length(int num_taps);

typedef struct thread_fft_cache_tag
{
	int * lsx_fft_br;
	double * lsx_fft_sc;
	size_t fft_len;

	size_t br_size, sc_size;
}thread_fft_cache;

void init_fft_cache(thread_fft_cache* info);
void update_fft_cache(size_t len, thread_fft_cache* info);
void free_fft_cache(thread_fft_cache* info);

static __inline void lsx_safe_rdft_SSE3(size_t len, int type, double * d, thread_fft_cache* info)
{
  if (len > info->fft_len) update_fft_cache(len, info);
  lsx_rdft_SSE3(len, type, d, info->lsx_fft_br, info->lsx_fft_sc);
}

static __inline void lsx_safe_rdft_generic(size_t len, int type, double * d, thread_fft_cache* info)
{
  if (len > info->fft_len) update_fft_cache(len, info);
  lsx_rdft_generic(len, type, d, info->lsx_fft_br, info->lsx_fft_sc);
}

static __inline void lsx_safe_rdft(size_t len, int type, double * d, thread_fft_cache* info)
{
  if (len > info->fft_len) update_fft_cache(len, info);
  lsx_rdft(len, type, d, info->lsx_fft_br, info->lsx_fft_sc);
}

double lsx_kaiser_beta(double att);
void lsx_apply_kaiser(double h[], const int num_points, double beta);
double * lsx_make_lpf(int num_taps, double Fc, double beta, double scale, sox_bool dc_norm);
int lsx_lpf_num_taps(double att, double tr_bw, int k);
double * lsx_design_lpf(
    double Fp,      /* End of pass-band; ~= 0.01dB point */
    double Fc,      /* Start of stop-band */
    double Fn,      /* Nyquist freq; e.g. 0.5, 1, PI */
    sox_bool allow_aliasing,
    double att,     /* Stop-band attenuation in dB */
    int * num_taps, /* (Single phase.)  0: value will be estimated */
    int k);         /* Number of phases; 0 for single-phase */
void lsx_fir_to_phase(double * * h, int * len,
    int * post_len, double phase0, thread_fft_cache* info);
#define LSX_TO_6dB .5869
#define LSX_TO_3dB ((2/3.) * (.5 + LSX_TO_6dB))
#define LSX_MAX_TBW0 36.
#define LSX_MAX_TBW0A (LSX_MAX_TBW0 / (1 + LSX_TO_3dB))
#define LSX_MAX_TBW3 floor(LSX_MAX_TBW0 * LSX_TO_3dB)
#define LSX_MAX_TBW3A floor(LSX_MAX_TBW0A * LSX_TO_3dB)


#endif
